local addonInfo, privateVars = ...

if nkAdvisor ~= nil then return end
nkAdvisor = {}

privateVars.debug = false

privateVars.modules = {}

privateVars.uiElements = {}
privateVars.uiElements.context = UI.CreateContext("nkAdvisor")
privateVars.uiElements.context:SetStrata ('dialog')

privateVars.uiElements.dialogContext = UI.CreateContext("nkAdvisor.Dialogs")
privateVars.uiElements.dialogContext:SetStrata ('topmost')

privateVars.uiElements.tooltipContext = UI.CreateContext("nkAdvisor.Tooltip")
privateVars.uiElements.tooltipContext:SetStrata ('topmost')

nkAdvisor.layout = {
	elementColor = { 0.925, 0.894, 0.741, 1 },
	labelColor = { 0.925, 0.894, 0.741, 1 }
}

function nkAdvisor.main(_, addon)

	if (addon == addonInfo.toc.Identifier) then

		if nkPanel ~= nil and nkAdvisorSettings.nkPanel == true then
			
			EnKai.art.SetTheme('default');  
			
			local items = {
			{ label = nkAdvisor.langTexts.menuShowWindow, callBack = function () nkAdvisor.Show() end},
			{ seperator = true }
			}		
		
			privateVars.uiElements.panel = nkAdvisor.nkPanelPlugin(items)
			nkPanel.api.registerPlugin('nkAdvisor main plugin', privateVars.uiElements.panel)
			privateVars.uiElements.panel:SetTitle("nkAdvisor")			
		else			
			EnKai.manager.init('nkAdvisor', nil, function () nkAdvisor.Show() end)
		end

		EnKai.version.init(addonInfo.toc.Identifier, addonInfo.toc.Version)
	end
	
end

function nkAdvisor.commandHandler(commandline)

	if string.find(commandline, "itemloader reset") then
		privateVars.itemList = {}
		privateVars.itemLoader = false
	elseif string.find(commandline, "itemupgrader reset") then
		privateVars.itemUpgradeList = {}
		privateVars.itemUpgrader = false
	end
	
	if string.find(commandline, "itemloader") then
		if privateVars.uiElements.itemLoader == nil then privateVars.uiElements.itemLoader = nkAdvisor.itemLoaderUI() end

		if privateVars.itemLoader == true then
			privateVars.itemLoader = false
			Command.Event.Detach(Event.Tooltip, nil, "nkAdvisor.Tooltip")
			privateVars.uiElements.itemLoader:SetVisible(false)
		else
			privateVars.itemLoader = true
			Command.Event.Attach(Event.Tooltip, nkAdvisor.itemLoader, "nkAdvisor.Tooltip")
			privateVars.uiElements.itemLoader:SetVisible(true)
		end
	elseif string.find(commandline, "itemupgrader") then
		if privateVars.uiElements.itemUpgrader == nil then privateVars.uiElements.itemUpgrader = nkAdvisor.itemUpgraderUI() end

		if privateVars.itemUpgrader == true then
			privateVars.itemUpgrader = false
			Command.Event.Detach(Event.Tooltip, nil, "nkAdvisor.Tooltip")
			privateVars.uiElements.itemUpgrader:SetVisible(false)
		else
			privateVars.itemUpgrader = true
			Command.Event.Attach(Event.Tooltip, nkAdvisor.itemUpgrader, "nkAdvisor.Tooltip")
			privateVars.uiElements.itemUpgrader:SetVisible(true)
		end
	else

		if privateVars.uiElements.window == nil then 
			privateVars.uiElements.window = nkAdvisor.UI()
		end
		
		privateVars.uiElements.window:SetVisible(true)
	end
		
end

function nkAdvisor.settingsHandler(_, addon) 

	if addon == addonInfo.identifier then
		if nkAdvisorSettings == nil then nkAdvisorSettings = { nkPanel = true } end
	end
	
	if nkAdvisorSettings.nkPanel == nil then nkAdvisorSettings.nkPanel = true end
					
end

-- sub routines

function nkAdvisor.itemLoader(_, tttype, shown, buff)
	
	if tttype ~= 'item' and tttype ~= 'itemtype' then return end
	
	if privateVars.itemList == nil then privateVars.itemList = {} end
	
	local key = nkItemBase.Query.byRiftInternalKey (shown, nil)
	
	if key == nil then
	  local details = Inspect.Item.Detail(shown)
	  key = details.type  
	end
	
	if EnKai.tools.table.isMember(privateVars.itemList, key) == false then
		table.insert(privateVars.itemList, key)
		privateVars.uiElements.itemLoader.listField:SetText (EnKai.tools.table.serialize (privateVars.itemList))
	end		
		
end

function nkAdvisor.itemUpgrader(_, tttype, shown, buff)
	
	if tttype ~= 'item' and tttype ~= 'itemtype' then return end
	
	if privateVars.itemUpgradeList == nil then privateVars.itemUpgradeList = {} end
	
	local key = nkItemBase.Query.byRiftInternalKey (shown, nil)
	
	if key == nil then return end
	
	if privateVars.firstItem == key then return end

	local details = Inspect.Item.Detail(shown)
		
	if privateVars.firstItem == nil then 
		privateVars.firstItem = key
		privateVars.uiElements.itemUpgrader.firstItem:SetText(details.name)
		return
	end
	
	privateVars.secondItem = key
	privateVars.uiElements.itemUpgrader.secondItem:SetText(details.name)
		
end


function nkAdvisor.UnitAvailable()

	 nkAdvisor.getPlayerDetails ()

end

Command.Event.Attach(Event.Addon.Load.End, nkAdvisor.main, "nkAdvisor.Addon.Load.End")
Command.Event.Attach(Event.Unit.Availability.Full, nkAdvisor.UnitAvailable, "nkAdvisor.Unit.Availability.Full")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, nkAdvisor.settingsHandler, "nkAdvisor.SavedVariables.Load.End")

table.insert(Command.Slash.Register("nka"), {nkAdvisor.commandHandler, "nkAdvisor", "config"})

 -- check of gebraucht wird

function nkAdvisor.getPlayerDetails ()

	if privateVars.activeCalling == nil then	

		local userDetails = Inspect.Unit.Detail ('player')
		if userDetails == nil then 
			nkAAPI.showErrorDialog (nkAdvisor.langTexts.messages.errGettingPlayerDetails)
			return false 
		end
		
		privateVars.activeCalling = userDetails.calling
		privateVars.faction = userDetails.faction
		
		if privateVars.faction == nil then privateVars.faction = 'defiant' end
		
	end

	return true

end

-- ========================================== ITEM LOADER UI =============================================

function nkAdvisor.itemLoaderUI()

	local ui = EnKai.uiCreateFrame("nkWindowElement", "nkAdvisor.itemLoader", privateVars.uiElements.context)
	ui:SetWidth(400)
	ui:SetHeight(200)
	ui:SetTitle(addonInfo.toc.Name)
	ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 300, 300)
	ui:SetDragable(true)
  ui:SetCloseable(true)
  
  ui.button = EnKai.uiCreateFrame("nkButtonMetro", "nkAdvisor.itemLoader.clearButton", ui:GetContent())
  ui.button:SetPoint("TOPLEFT", ui:GetContent(), "TOPLEFT", 10, 5)
  ui.button:SetText("Löschen")
  
  ui.button:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    ui.listField:SetText("")
    privateVars.itemList = nil
  end, "nkAdvisor.itemUpgrader.storeButton.Click")
	
	ui.listField = EnKai.uiCreateFrame("nkTextfield", 'nkAdvisor.itemLoader.listField', ui:GetContent())
	ui.listField:SetPoint("BOTTOMCENTER", ui:GetContent(), "BOTTOMCENTER", 0, -10)
	ui.listField:SetWidth(350)
	ui.listField:SetHeight(120)
		
	return ui
	
end

-- ========================================== ITEM UPGRADER UI =============================================

function nkAdvisor.itemUpgraderUI()

	local ui = EnKai.uiCreateFrame("nkWindowElement", "nkAdvisor.itemUpgrader", privateVars.uiElements.context)
	ui:SetWidth(400)
	ui:SetHeight(200)
	ui:SetTitle(addonInfo.toc.Name)
	ui:SetPoint("CENTER", UIParent, "CENTER")
	ui:SetDragable(true)
	ui:SetCloseable(true)
	
	ui.firstItem = UI.CreateFrame("Text", "nkAdvisor.itemUpgrader.key1", ui)
	ui.firstItem:SetPoint("TOPLEFT", ui, "TOPLEFT", 10, 50)
	
	ui.secondItem = UI.CreateFrame("Text", "nkAdvisor.itemUpgrader.key2", ui)
	ui.secondItem:SetPoint("TOPLEFT", ui.firstItem, "BOTTOMLEFT", 0, 5)
	
	ui.thirdItem = UI.CreateFrame("Text", "nkAdvisor.itemUpgrader.key3", ui)
	ui.thirdItem:SetPoint("TOPLEFT", ui.secondItem, "BOTTOMLEFT", 0, 5)
	
	ui.listField = EnKai.uiCreateFrame("nkTextfield", 'nkAdvisor.itemUpgrader.listField', ui)
	ui.listField:SetPoint("BOTTOMCENTER", ui, "BOTTOMCENTER", 0, -10)
	ui.listField:SetWidth(350)
	ui.listField:SetHeight(30)
	
	ui.button = UI.CreateFrame("RiftButton", "nkAdvisor.itemUpgrader.storeButton", ui)
	ui.button:SetText("add")
	ui.button:SetPoint("TOPRIGHT", ui, "TOPRIGHT", -20, 50)
	
	ui.button:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
		local upgradeText = "[\"" .. privateVars.firstItem .. "\"]=\"" .. privateVars.secondItem .. "\","
	
		if EnKai.tools.table.isMember(privateVars.itemUpgradeList, upgradeText) == false then
			table.insert(privateVars.itemUpgradeList, upgradeText)
			privateVars.uiElements.itemUpgrader.listField:SetText (EnKai.tools.table.serialize (privateVars.itemUpgradeList))
		end		
		
		privateVars.firstItem = nil
		privateVars.secondItem = nil
		
		ui.firstItem:SetText("")
		ui.secondItem:SetText("")
	end, "nkAdvisor.itemUpgrader.storeButton.Click")
	
	ui.resetbutton = UI.CreateFrame("RiftButton", "nkAdvisor.itemUpgrader.resetButton", ui)
	ui.resetbutton:SetText("reset")
	ui.resetbutton:SetPoint("TOPCENTER", ui.button, "BOTTOMCENTER", 0, 10)
	
	ui.resetbutton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
		privateVars.firstItem = nil
		privateVars.secondItem = nil
	
		ui.firstItem:SetText("")
		ui.secondItem:SetText("")
	end, "nkAdvisor.itemUpgrader.resetButton.Click")
		
	return ui
	
end

-- ========================================== USER INTERFACE =============================================

function nkAdvisor.UI ()

  local name = "nkAdvisor.ui"

	local ui = EnKai.uiCreateFrame("nkWindowMetro", name, privateVars.uiElements.context)
	ui:SetWidth(1055)
	ui:SetHeight(600)
	ui:SetTitle(addonInfo.toc.Name)
	ui:SetPoint("CENTER", UIParent, "CENTER")
				
	local tabPane = EnKai.uiCreateFrame("nkTabpaneMetro", name .. ".tabPane", ui:GetContent())
	tabPane:SetPoint("TOPLEFT", ui:GetContent(), "TOPLEFT", 10, 10)
  tabPane:SetPoint("BOTTOMRIGHT", ui:GetContent(), "BOTTOMRIGHT", -10, -50)
  tabPane:SetLayer(1)
  tabPane:SetBorder(false)
	
	local panes = {}
	nkAAPI.uiElements.panes = {}
	
	 for idx = 1, 9, 1 do
		for idx2 = 1, #privateVars.modules, 1 do
			if privateVars.modules[idx2].set == idx then		
				local frame = UI.CreateFrame ('Frame', name .. '.ui.tab' .. idx2, tabPane)
				frame:SetVisible(false)
				frame:SetPoint("TOPLEFT", tabPane, "TOPLEFT", 10, 10)
				frame:SetPoint("BOTTOMRIGHT", tabPane, "BOTTOMRIGHT", -10, -10)
				frame:SetLayer(3)
				
				table.insert (panes, {content = frame, label = privateVars.modules[idx2].label, initFunc = function () privateVars.modules[idx2].func(frame, ui) end })
				nkAAPI.uiElements.panes[privateVars.modules[idx2].name] = frame
			end
		end
	end
	
	if #panes == 0 then
		local noModules = UI.CreateFrame('Frame', name .. '.tabNoModule', tabPane)
		
		function noModules:setupModules() 
			local noModulesText = UI.CreateFrame("Text", 'nkAdvisor.ui.tabNoModule.noModulesText', noModules)
			noModulesText:SetPoint("TOPCENTER", noModules, "TOPCENTER", 2, 102)
			noModulesText:SetEffectGlow({ offsetX = 2, offsetY = 2})
			noModulesText:SetText(nkAdvisor.langTexts.modules.errNoModules)
			noModulesText:SetFontSize(30)
			noModulesText:SetFontColor(0.906, 0.784, 0.471, 1)	
			
			local modulesRequiredText = UI.CreateFrame("Text", name .. '.tabNoModule.modulesRequiredText', noModules)
			modulesRequiredText:SetPoint("TOPCENTER", noModulesText, "BOTTOMCENTER", 0, 20)
			modulesRequiredText:SetEffectGlow({ offsetX = 2, offsetY = 2})
			modulesRequiredText:SetText(nkAdvisor.langTexts.modules.msgModulesRequired)
			modulesRequiredText:SetFontSize(30)
			modulesRequiredText:SetFontColor(0.906, 0.784, 0.471, 1)	
			
			local loadModulesText = UI.CreateFrame("Text", name .. '.tabNoModule.loadModulesText', noModules)
			loadModulesText:SetPoint("TOPCENTER", modulesRequiredText, "BOTTOMCENTER", 0, 20)
			loadModulesText:SetEffectGlow({ offsetX = 2, offsetY = 2})
			loadModulesText:SetText(nkAdvisor.langTexts.modules.msgDownload)
			loadModulesText:SetFontSize(20)
			loadModulesText:SetFontColor(0.906, 0.784, 0.471, 1)	
			
		end
		
		noModules:SetVisible(false)				
		table.insert (panes, {content = noModules, label = nkAdvisor.langTexts.tabModules, initFunc = function () noModules:setupModules() end} )
	end
	
	for idx = 1, #panes, 1 do
		local flag = false
		if idx == #panes then flag = true end
		tabPane:AddPane( { label = panes[idx].label, frame = panes[idx].content, initFunc = panes[idx].initFunc}, flag)
	end
		
	function ui:GetTabPane()
		return tabPane
	end
	
	local EnKaiLogo = EnKai.uiCreateFrame("nkTexture", name .. ".EnKaiLogo", ui)
  EnKaiLogo:SetTextureAsync(EnKai.art.GetThemeLogo()[1], EnKai.art.GetThemeLogo()[2])
  EnKaiLogo:SetPoint("BOTTOMLEFT", ui:GetContent(), "BOTTOMLEFT", 10, -5)
  EnKaiLogo:SetWidth(125)
  EnKaiLogo:SetHeight(33)
  
  local closeButton = EnKai.uiCreateFrame("nkButtonMetro", name .. ".closeButton", ui)
  
  closeButton:SetPoint("BOTTOMRIGHT", ui:GetContent(), "BOTTOMRIGHT", -10, -10)
  closeButton:SetText(nkAdvisor.langTexts.btClose)
  closeButton:SetIcon("nkAdvisor", "gfx/iconClose.png")
  closeButton:SetScale(.8)
  closeButton:SetLayer(9)
  
  Command.Event.Attach(EnKai.events[name .. ".closeButton"].Clicked, function (_, newValue)
    ui:SetVisible(false) 
  end, name .. ".closeButton.Clicked")
	
	local versionText = UI.CreateFrame("Text", name .. ".versionText", ui)
  versionText:SetFontSize(11)
  versionText:SetText(string.format(nkAdvisor.langTexts.txtVersion, addonInfo.toc.Version))
  versionText:SetPoint("BOTTOMRIGHT", tabPane, "BOTTOMRIGHT", -5, -5)
  versionText:SetLayer(99)
		
	return ui

end

function nkAdvisor.Show()

	if privateVars.uiElements.window == nil then 
		privateVars.uiElements.window = nkAdvisor.UI()
	elseif privateVars.uiElements.window:GetVisible() == true then
		privateVars.uiElements.window:SetVisible(false)
	else
		privateVars.uiElements.window:SetVisible(true)
	end

end